/* avagl: sandbox.cc
 * 
 * Copyright 2003 Chun-Chung Chen <cjj@u.washington.edu>
 * 
 * This program is free software; you can redistribute it and'or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 */

#include "random.hh"
#include "sandbox.hh"
#include <iostream>

#define site(x, y) sys[(y) * width + (x)]
#define S(x, y) ((y) * width + (x))
#define support(x, y) min(site(x, (y) - 1), site(((x) + 1) % width, (y) - 1))
template <typename T> inline T min(T x, T y) {return (x < y) ? x : y;}
#define eta() ((R() > p) ? 0 : 1)


Sandbox::Sandbox() : sys(0)
{
}

Sandbox::~Sandbox()
{
	if (sys != 0) {
		delete[] sys;
		delete[] last_active;
	}
}

void Sandbox::initialize(unsigned w, unsigned l)
{
	width = w;
	length = l;
	if (sys != 0) delete[] sys;
	sys = new int[width * length];
	last_active = new bool[width * length];
	R.init();
	p = 0.5;
}

void Sandbox::start()
{
	for (unsigned x = 0; x < width; x ++) site(x, 0) = int(R() + 0.5);
	for (unsigned y = 1; y < length; y ++) for (unsigned x = 0; x < width; x ++) {
		site(x, y) = support(x, y) + eta();
		last_active[S(x, y)] = true;
	}
	count = 0;
}

void Sandbox::iterate()
{
	unsigned i = 0;
	int maxv = -100;
	int ma[width];
	unsigned na = 0;

	// find all sites at maximum height
	for (unsigned x = 0; x < width; x ++) if (site(x, 0) >= maxv) {
		if (site(x, 0) > maxv) {
			maxv = site(x, 0);
			na = 0;
		}
		ma[na] = x;
		na ++;
	}

	// shift up the surface if necessary
	if (maxv < 2) for (unsigned x = 0; x < width; x ++)
		for (unsigned y = 0; y < length; y ++) site(x, y) += 2 - maxv;
	for (unsigned x = 0; x < width; x ++)
		for (unsigned y = 0; y < length; y ++)
			last_active[S(x, y)] = false;

	// randomly choose a site of maximum height
	i = ma[unsigned(R() * na)];

	// topple the site
	site(i, 0) += eta() - 2;
	last_active[S(i, 0)] = true;

	// topple unstable sites row by row
	for (unsigned y = 1; y < length; y ++) {
		bool active = false;
		for (unsigned x = 0; x < width; x ++) {
			int s = support(x, y);
			if (site(x, y) - s > 1) {
				site(x, y) = s + eta();
				last_active[S(x, y)] = true;
				active = true;
			}
		}
		if (! active) break;
	}
	count ++;
}

unsigned long Sandbox::get_count()
{
	return count;
}

const int * Sandbox::get_prof()
{
	return sys;
}

const bool * Sandbox::get_active()
{
	return last_active;
}

void Sandbox::set_prob(double pr)
{
	p = pr;
}
